@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-zoom-control.c 2571 2007-07-11 16:22:43Z carlosgc $
+ *  $Id: ephy-zoom-control.c 3240 2008-10-27 12:17:08Z chpe $
  */
 
 #include "config.h"
@@ -26,10 +26,6 @@
 #include "ephy-zoom.h"
 
 #include <gtk/gtk.h>
-#include <gtk/gtkcombobox.h>
-#if !GTK_CHECK_VERSION (2, 11, 5)
-#include <gtk/gtktooltips.h>
-#endif
 #include <glib/gi18n.h>
 
 #define EPHY_ZOOM_CONTROL_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_ZOOM_CONTROL, EphyZoomControlPrivate))
@@ -177,8 +173,7 @@ ephy_zoom_control_init (EphyZoomControl *control)
 					      NULL, NULL);
 
 	gtk_combo_box_set_focus_on_click (p->combo, FALSE);
-	g_object_ref (p->combo);
-	gtk_object_sink (GTK_OBJECT (p->combo));
+	g_object_ref_sink (G_OBJECT (p->combo));
 	gtk_widget_show (GTK_WIDGET (p->combo));
 
 	i = ephy_zoom_get_zoom_level_index (p->zoom);
