@@ -21,45 +21,47 @@
  */
 
 #include <config.h>
-#include "ev-application.h"
-#include "ev-utils.h"
-#include "ev-file-helpers.h"
-#include "ev-document-factory.h"
-#ifdef ENABLE_DBUS
-#include "ev-media-player-keys.h"
-#endif /* ENABLE_DBUS */
-#include "totem-scrsaver.h"
-
-#include <glib.h>
-#include <glib/gi18n.h>
-#include <glib-object.h>
-#include <gtk/gtkfilechooserdialog.h>
-#include <gtk/gtkstock.h>
-#include <gtk/gtkwidget.h>
-#include <gtk/gtkmain.h>
-
-#if WITH_GNOME
-#include <libgnomeui/gnome-client.h>
-#endif
-
-#ifdef ENABLE_DBUS
-#include "ev-application-service.h"
-#include <dbus/dbus-glib-bindings.h>
-#endif
 
 #include <string.h>
 
-static void ev_application_add_icon_path_for_screen (GdkScreen *screen);
+#include <glib.h>
+#include <glib/gi18n.h>
+#include <gtk/gtk.h>
+#include <gdk/gdkx.h>
+
+#include "totem-scrsaver.h"
+#include "eggsmclient.h"
+
+#include "ev-application.h"
+#include "ev-document-factory.h"
+#include "ev-file-helpers.h"
+#include "ev-utils.h"
+
+#ifdef ENABLE_DBUS
+#include "ev-media-player-keys.h"
+#endif /* ENABLE_DBUS */
+
+#ifdef ENABLE_DBUS
+#include <dbus/dbus-glib-bindings.h>
+#include "ev-application-service.h"
+#endif
+
+static void ev_application_add_icon_path_for_screen (GdkScreen     *screen);
+static void ev_application_save_print_settings      (EvApplication *application);
 
 struct _EvApplication {
 	GObject base_instance;
 
+	gchar *accel_map_file;
+	
 	gchar *toolbars_file;
 
 	EggToolbarsModel *toolbars_model;
 
 	TotemScrsaver *scr_saver;
 
+	EggSMClient *smclient;
+
 	gchar *last_chooser_uri;
 
 #ifdef ENABLE_DBUS
@@ -67,9 +69,8 @@ struct _EvApplication {
 #endif /* ENABLE_DBUS */
 
 	GtkPrintSettings *print_settings;
-#if GTK_CHECK_VERSION (2, 11, 0)
-	gchar            *print_settings_file;
-#endif
+	GtkPageSetup     *page_setup;
+	GKeyFile         *print_settings_file;
 };
 
 struct _EvApplicationClass {
@@ -80,6 +81,10 @@ G_DEFINE_TYPE (EvApplication, ev_application, G_TYPE_OBJECT);
 
 #define APPLICATION_SERVICE_NAME "org.gnome.evince.ApplicationService"
 
+#define EV_PRINT_SETTINGS_FILE "print-settings"
+#define EV_PRINT_SETTINGS_GROUP "Print Settings"
+#define EV_PAGE_SETUP_GROUP "Page Setup"
+
 #ifdef ENABLE_DBUS
 gboolean
 ev_application_register_service (EvApplication *application)
@@ -152,55 +157,89 @@ ev_application_get_instance (void)
 	return instance;
 }
 
-#if WITH_GNOME
-static void
-removed_from_session (GnomeClient *client, EvApplication *application)
+/* Session */
+gboolean
+ev_application_load_session (EvApplication *application)
 {
-	ev_application_shutdown (application);
-}
-
-static gint
-save_session (GnomeClient *client, gint phase, GnomeSaveStyle save_style, gint shutdown,
-	      GnomeInteractStyle interact_style, gint fast, EvApplication *application)
-{
-	GList *windows, *l;
-	char **restart_argv;
-	int argc = 0, k;
-
-	windows = ev_application_get_windows (application);
-	restart_argv = g_new (char *, g_list_length (windows) + 1);
-	restart_argv[argc++] = g_strdup ("evince");
-
-	for (l = windows; l != NULL; l = l->next) {
-		EvWindow *window = EV_WINDOW (l->data);
-		restart_argv[argc++] = g_strdup (ev_window_get_uri (window));
-	}
-
-	gnome_client_set_restart_command (client, argc, restart_argv);
-
-	for (k = 0; k < argc; k++) {
-		g_free (restart_argv[k]);
-	}
-
-	g_list_free (windows);
-	g_free (restart_argv);
+	GKeyFile *state_file;
+	gchar   **uri_list;
 	
+	if (!egg_sm_client_is_resumed (application->smclient))
+		return FALSE;
+
+	state_file = egg_sm_client_get_state_file (application->smclient);
+	if (!state_file)
+		return FALSE;
+
+	uri_list = g_key_file_get_string_list (state_file,
+					       "Evince",
+					       "documents",
+					       NULL, NULL);
+	if (uri_list) {
+		gint i;
+
+		for (i = 0; uri_list[i]; i++) {
+			if (g_ascii_strcasecmp (uri_list[i], "empty-window") == 0)
+				ev_application_open_window (application, NULL, GDK_CURRENT_TIME, NULL);
+			else
+				ev_application_open_uri (application, uri_list[i], NULL, GDK_CURRENT_TIME, NULL);
+		}
+		g_strfreev (uri_list);
+	}
+	g_key_file_free (state_file);
+
 	return TRUE;
 }
 
 static void
-init_session (EvApplication *application)
+smclient_save_state_cb (EggSMClient   *client,
+			GKeyFile      *state_file,
+			EvApplication *application)
 {
-	GnomeClient *client;
+	GList *windows, *l;
+	gint i;
+	const gchar **uri_list;
+	const gchar *empty = "empty-window";
 
-	client = gnome_master_client ();
+	windows = ev_application_get_windows (application);
+	if (!windows)
+		return;
 
-	g_signal_connect (client, "save_yourself",
-			  G_CALLBACK (save_session), application);	
-	g_signal_connect (client, "die",
-			  G_CALLBACK (removed_from_session), application);
+	uri_list = g_new (const gchar *, g_list_length (windows));
+	for (l = windows, i = 0; l != NULL; l = g_list_next (l), i++) {
+		EvWindow *window = EV_WINDOW (l->data);
+
+		if (ev_window_is_empty (window))
+			uri_list[i] = empty;
+		else
+			uri_list[i] = ev_window_get_uri (window);
+	}
+	g_key_file_set_string_list (state_file,
+				    "Evince",
+				    "documents", 
+				    (const char **)uri_list,
+				    i);
+	g_free (uri_list);
+}
+
+static void
+smclient_quit_cb (EggSMClient   *client,
+		  EvApplication *application)
+{
+	ev_application_shutdown (application);
+}
+
+static void
+ev_application_init_session (EvApplication *application)
+{
+	application->smclient = egg_sm_client_get ();
+	g_signal_connect (application->smclient, "save_state",
+			  G_CALLBACK (smclient_save_state_cb),
+			  application);
+	g_signal_connect (application->smclient, "quit",
+			  G_CALLBACK (smclient_quit_cb),
+			  application);
 }
-#endif
 
 /**
  * ev_display_open_if_needed:
@@ -412,11 +451,20 @@ ev_application_open_window (EvApplication  *application,
 		gtk_window_set_screen (GTK_WINDOW (new_window), screen);
 	}
 	ev_application_add_icon_path_for_screen (screen);
+
+	if (!GTK_WIDGET_REALIZED (new_window))
+		gtk_widget_realize (new_window);
 	
-	gtk_widget_show (new_window);
+#ifdef GDK_WINDOWING_X11
+	if (timestamp <= 0)
+		timestamp = gdk_x11_get_server_time (GTK_WIDGET (new_window)->window);
+	gdk_x11_window_set_user_time (GTK_WIDGET (new_window)->window, timestamp);
 	
-	gtk_window_present_with_time (GTK_WINDOW (new_window),
-				      timestamp);
+	gtk_window_present (GTK_WINDOW (new_window));
+#else
+	gtk_window_present_with_time (GTK_WINDOW (new_window), timestamp);
+#endif /* GDK_WINDOWING_X11 */
+
 	return TRUE;
 }
 
@@ -496,7 +544,7 @@ ev_application_add_icon_path_for_screen (GdkScreen *screen)
 {
 	GtkIconTheme *icon_theme;
 
-	icon_theme = gtk_icon_theme_get_for_screen (screen);
+	icon_theme = screen ? gtk_icon_theme_get_for_screen (screen) : gtk_icon_theme_get_default ();
 	if (icon_theme) {
 		gchar **path = NULL;
 		gint    n_paths;
@@ -567,12 +615,22 @@ ev_application_open_uri_at_dest (EvApplication  *application,
 	ev_window_open_uri (new_window, uri, dest, mode, search_string, 
 			    unlink_temp_file, print_settings);
 
-	ev_document_fc_mutex_lock ();
-	gtk_widget_show (GTK_WIDGET (new_window));
-	ev_document_fc_mutex_unlock ();
+	if (!GTK_WIDGET_REALIZED (GTK_WIDGET (new_window)))
+		gtk_widget_realize (GTK_WIDGET (new_window));
 
-	gtk_window_present_with_time (GTK_WINDOW (new_window),
-				      timestamp);
+#ifdef GDK_WINDOWING_X11
+	if (timestamp <= 0)
+		timestamp = gdk_x11_get_server_time (GTK_WIDGET (new_window)->window);
+	gdk_x11_window_set_user_time (GTK_WIDGET (new_window)->window, timestamp);
+
+	ev_document_fc_mutex_lock ();
+	gtk_window_present (GTK_WINDOW (new_window));
+	ev_document_fc_mutex_unlock ();
+#else
+	ev_document_fc_mutex_lock ();
+	gtk_window_present_with_time (GTK_WINDOW (new_window), timestamp);
+	ev_document_fc_mutex_unlock ();
+#endif /* GDK_WINDOWING_X11 */
 }
 
 /**
@@ -636,6 +694,12 @@ ev_application_open_uri_list (EvApplication *application,
 void
 ev_application_shutdown (EvApplication *application)
 {
+	if (application->accel_map_file) {
+		gtk_accel_map_save (application->accel_map_file);
+		g_free (application->accel_map_file);
+		application->accel_map_file = NULL;
+	}
+	
 	if (application->toolbars_model) {
 		g_object_unref (application->toolbars_model);
 		g_free (application->toolbars_file);
@@ -643,32 +707,29 @@ ev_application_shutdown (EvApplication *application)
 		application->toolbars_file = NULL;
 	}
 
-#if GTK_CHECK_VERSION (2, 11, 0)
+	ev_application_save_print_settings (application);
+	
 	if (application->print_settings_file) {
-		if (application->print_settings) {
-			GError *error = NULL;
-			
-			gtk_print_settings_to_file (application->print_settings,
-						    application->print_settings_file,
-						    &error);
-			if (error) {
-				g_warning ("%s", error->message);
-				g_error_free (error);
-			}
-
-			g_object_unref (application->print_settings);
-			application->print_settings = NULL;
-		}
-
-		g_free (application->print_settings_file);
+		g_key_file_free (application->print_settings_file);
 		application->print_settings_file = NULL;
 	}
-#else /* ! GTK 2.11.0 */
+
 	if (application->print_settings) {
 		g_object_unref (application->print_settings);
 		application->print_settings = NULL;
 	}
-#endif /* GTK 2.11.0 */
+
+	if (application->page_setup) {
+		g_object_unref (application->page_setup);
+		application->page_setup = NULL;
+	}
+
+#ifdef ENABLE_DBUS
+	if (application->keys) {
+		g_object_unref (application->keys);
+		application->keys = NULL;
+	}
+#endif /* ENABLE_DBUS */
 	
 	g_free (application->last_chooser_uri);
 	g_object_unref (application);
@@ -685,11 +746,20 @@ static void
 ev_application_init (EvApplication *ev_application)
 {
 	gint i;
+	const gchar *home_dir;
 	
-#if WITH_GNOME
-	init_session (ev_application);
-#endif
+	ev_application_init_session (ev_application);
 
+	home_dir = g_get_home_dir ();
+	if (home_dir) {
+		ev_application->accel_map_file = g_build_filename (home_dir,
+								   ".gnome2",
+								   "accels"
+								   "evince",
+								   NULL);
+		gtk_accel_map_load (ev_application->accel_map_file);
+	}
+	
 	ev_application->toolbars_model = egg_toolbars_model_new ();
 
 	ev_application->toolbars_file = g_build_filename
@@ -763,13 +833,11 @@ ev_application_get_windows (EvApplication *application)
  *
  * Returns: A #EvMediaPlayerKeys.
  */
-GObject
-*ev_application_get_media_keys (EvApplication *application)
+GObject *
+ev_application_get_media_keys (EvApplication *application)
 {
 #ifdef ENABLE_DBUS
-	if (!application->keys)
-		return NULL;
-	return g_object_ref (G_OBJECT (application->keys));
+	return G_OBJECT (application->keys);
 #else
 	return NULL;
 #endif /* ENABLE_DBUS */
@@ -815,34 +883,83 @@ ev_application_screensaver_disable (EvApplication *application)
 		totem_scrsaver_disable (application->scr_saver);	
 }
 
-GtkPrintSettings *
-ev_application_get_print_settings (EvApplication *application)
+static GKeyFile *
+ev_application_get_print_settings_file (EvApplication *application)
 {
-	if (application->print_settings)
-		return application->print_settings;
+	gchar *filename;
 	
-#if GTK_CHECK_VERSION (2, 11, 0)
-	if (!application->print_settings_file) {
-		application->print_settings_file =
-			g_build_filename (ev_dot_dir (), "print-settings", NULL);
-	}
+	if (application->print_settings_file)
+		return application->print_settings_file;
 
-	if (g_file_test (application->print_settings_file, G_FILE_TEST_IS_REGULAR)) {
+	application->print_settings_file = g_key_file_new ();
+	
+	filename = g_build_filename (ev_dot_dir (), EV_PRINT_SETTINGS_FILE, NULL);
+	if (g_file_test (filename, G_FILE_TEST_IS_REGULAR)) {
 		GError *error = NULL;
-		
-		application->print_settings =
-			gtk_print_settings_new_from_file (application->print_settings_file, &error);
-		
+
+		g_key_file_load_from_file (application->print_settings_file,
+					   filename,
+					   G_KEY_FILE_KEEP_COMMENTS |
+					   G_KEY_FILE_KEEP_TRANSLATIONS,
+					   &error);
 		if (error) {
 			g_warning ("%s", error->message);
 			g_error_free (error);
-		} else {
-			return application->print_settings;
 		}
 	}
-#endif /* GTK 2.11.0 */
+	g_free (filename);
+
+	return application->print_settings_file;
+}
+
+static void
+ev_application_save_print_settings (EvApplication *application)
+{
+	GKeyFile *key_file;
+	gchar    *filename;
+	gchar    *data;
+	gssize    data_length;
+	GError   *error = NULL;
+
+	if (!application->print_settings && !application->page_setup)
+		return;
 	
-	application->print_settings = gtk_print_settings_new ();
+	key_file = ev_application_get_print_settings_file (application);
+	if (application->print_settings)
+		gtk_print_settings_to_key_file (application->print_settings,
+						key_file,
+						EV_PRINT_SETTINGS_GROUP);
+	if (application->page_setup)
+		gtk_page_setup_to_key_file (application->page_setup,
+					    key_file,
+					    EV_PAGE_SETUP_GROUP);
+	
+	filename = g_build_filename (ev_dot_dir (), EV_PRINT_SETTINGS_FILE, NULL);
+	data = g_key_file_to_data (key_file, (gsize *)&data_length, NULL);
+	g_file_set_contents (filename, data, data_length, &error);
+	if (error) {
+		g_warning ("%s", error->message);
+		g_error_free (error);
+	}
+	g_free (data);
+	g_free (filename);
+}
+
+GtkPrintSettings *
+ev_application_get_print_settings (EvApplication *application)
+{
+	GKeyFile         *key_file;
+	GtkPrintSettings *print_settings;
+	
+	if (application->print_settings)
+		return application->print_settings;
+
+	key_file = ev_application_get_print_settings_file (application);
+	print_settings = g_key_file_has_group (key_file, EV_PRINT_SETTINGS_GROUP) ? 
+		gtk_print_settings_new_from_key_file (key_file, EV_PRINT_SETTINGS_GROUP, NULL) :
+		gtk_print_settings_new ();
+
+	application->print_settings = print_settings ? print_settings : gtk_print_settings_new ();
 
 	return application->print_settings;
 }
@@ -851,14 +968,57 @@ void
 ev_application_set_print_settings (EvApplication    *application,
 				   GtkPrintSettings *settings)
 {
+	GKeyFile *key_file;
+	
 	g_return_if_fail (GTK_IS_PRINT_SETTINGS (settings));
 	
 	if (settings == application->print_settings)
 		return;
 
+	key_file = ev_application_get_print_settings_file (application);
+	
 	if (application->print_settings)
 		g_object_unref (application->print_settings);
 	
 	application->print_settings = g_object_ref (settings);
+	gtk_print_settings_to_key_file (settings, key_file, EV_PRINT_SETTINGS_GROUP);
 }
 
+GtkPageSetup *
+ev_application_get_page_setup (EvApplication *application)
+{
+	GKeyFile     *key_file;
+	GtkPageSetup *page_setup;
+	
+	if (application->page_setup)
+		return application->page_setup;
+
+	key_file = ev_application_get_print_settings_file (application);
+	page_setup = g_key_file_has_group (key_file, EV_PAGE_SETUP_GROUP) ? 
+		gtk_page_setup_new_from_key_file (key_file, EV_PAGE_SETUP_GROUP, NULL) :
+		gtk_page_setup_new ();
+
+	application->page_setup = page_setup ? page_setup : gtk_page_setup_new ();
+
+	return application->page_setup;
+}
+
+void
+ev_application_set_page_setup (EvApplication *application,
+			       GtkPageSetup  *page_setup)
+{
+	GKeyFile *key_file;
+	
+	g_return_if_fail (GTK_IS_PAGE_SETUP (page_setup));
+	
+	if (page_setup == application->page_setup)
+		return;
+
+	key_file = ev_application_get_print_settings_file (application);
+	
+	if (application->page_setup)
+		g_object_unref (application->page_setup);
+	
+	application->page_setup = g_object_ref (page_setup);
+	gtk_page_setup_to_key_file (page_setup, key_file, EV_PAGE_SETUP_GROUP);
+}
