@@ -27,14 +27,13 @@
 #include "config.h"
 #endif
 
-#include <glib/gerror.h>
-#include <glib-object.h>
-#include <gtk/gtkprintsettings.h>
-
-#include "ev-window.h"
+#include <glib.h>
+#include <gtk/gtk.h>
 
 #include "egg-toolbars-model.h"
 
+#include "ev-window.h"
+
 G_BEGIN_DECLS
 
 typedef struct _EvApplication EvApplication;
@@ -54,7 +53,7 @@ EvApplication    *ev_application_get_instance        (void);
 gboolean          ev_application_register_service    (EvApplication   *application);
 void	          ev_application_shutdown	     (EvApplication   *application);
 
-
+gboolean          ev_application_load_session        (EvApplication   *application);
 gboolean          ev_application_open_window         (EvApplication   *application,
 						      GHashTable      *args,
 						      guint32          timestamp,
@@ -90,6 +89,9 @@ void		  ev_application_screensaver_disable (EvApplication   *application);
 GtkPrintSettings *ev_application_get_print_settings  (EvApplication   *application);
 void              ev_application_set_print_settings  (EvApplication   *application,
 						      GtkPrintSettings *settings);
+GtkPageSetup     *ev_application_get_page_setup      (EvApplication   *application);
+void              ev_application_set_page_setup      (EvApplication   *application,
+						      GtkPageSetup    *page_setup);
 
 G_END_DECLS
 
