@@ -19,35 +19,30 @@
 
 #include "config.h"
 
-#include "ev-application.h"
-#include "ev-metadata-manager.h"
-
-#include <glib/gi18n.h>
-#include <gdk/gdkx.h>
-#include <gtk/gtkmain.h>
 #include <stdlib.h>
 #include <string.h>
 
-#if WITH_GNOME
-#include <libgnome/gnome-program.h>
-#include <libgnomeui/gnome-ui-init.h>
-#include <libgnomeui/gnome-app-helper.h>
-#include <libgnomeui/gnome-authentication-manager.h>
-#endif
+#include <glib/gi18n.h>
+#include <gtk/gtk.h>
 
 #ifdef ENABLE_DBUS
+#include <gdk/gdkx.h>
 #include <dbus/dbus-glib-bindings.h>
 #endif
 
-#include "ev-stock-icons.h"
-#include "ev-job-queue.h"
-#include "ev-file-helpers.h"
+#include "ev-application.h"
 #include "ev-backends-manager.h"
+#include "ev-debug.h"
+#include "ev-file-helpers.h"
+#include "ev-metadata-manager.h"
+#include "ev-stock-icons.h"
+#include "eggsmclient.h"
+#include "eggdesktopfile.h"
 
 static gchar   *ev_page_label;
 static gchar   *ev_find_string;
 static gboolean preview_mode = FALSE;
-static gboolean fullscren_mode = FALSE;
+static gboolean fullscreen_mode = FALSE;
 static gboolean presentation_mode = FALSE;
 static gboolean unlink_temp_file = FALSE;
 static gchar   *print_settings;
@@ -56,7 +51,7 @@ static const char **file_arguments = NULL;
 static const GOptionEntry goption_options[] =
 {
 	{ "page-label", 'p', 0, G_OPTION_ARG_STRING, &ev_page_label, N_("The page of the document to display."), N_("PAGE")},
-	{ "fullscreen", 'f', 0, G_OPTION_ARG_NONE, &fullscren_mode, N_("Run evince in fullscreen mode"), NULL },
+	{ "fullscreen", 'f', 0, G_OPTION_ARG_NONE, &fullscreen_mode, N_("Run evince in fullscreen mode"), NULL },
 	{ "presentation", 's', 0, G_OPTION_ARG_NONE, &presentation_mode, N_("Run evince in presentation mode"), NULL },
 	{ "preview", 'w', 0, G_OPTION_ARG_NONE, &preview_mode, N_("Run evince as a previewer"), NULL },
 	{ "find", 'l', 0, G_OPTION_ARG_STRING, &ev_find_string, N_("The word or phrase to find in the document"), N_("STRING")},
@@ -145,7 +140,7 @@ arguments_parse (void)
 		ev_page_label = NULL;
 	}
 
-	if (fullscren_mode)
+	if (fullscreen_mode)
 		mode = EV_WINDOW_MODE_FULLSCREEN;
 	else if (presentation_mode)
 		mode = EV_WINDOW_MODE_PRESENTATION;
@@ -251,7 +246,7 @@ load_files_remote (const char **files,
 	connection = dbus_g_bus_get (DBUS_BUS_STARTER, &error);
 
 	if (connection == NULL) {
-		g_warning (error->message);
+		g_warning ("%s", error->message);
 		g_error_free (error);	
 
 		return FALSE;
@@ -267,7 +262,7 @@ load_files_remote (const char **files,
 					G_TYPE_UINT, timestamp,
 					G_TYPE_INVALID,
 					G_TYPE_INVALID)) {
-			g_warning (error->message);
+			g_warning ("%s", error->message);
 			g_clear_error (&error);
 			g_object_unref (remote_object);
 			dbus_g_connection_unref (connection);
@@ -297,7 +292,7 @@ load_files_remote (const char **files,
 					G_TYPE_UINT, timestamp,
 					G_TYPE_INVALID,
 					G_TYPE_INVALID)) {
-			g_warning (error->message);
+			g_warning ("%s", error->message);
 			g_clear_error (&error);
 			g_free (uri);
 			continue;
@@ -319,94 +314,74 @@ load_files_remote (const char **files,
 int
 main (int argc, char *argv[])
 {
-	gboolean enable_metadata = FALSE;
 	GOptionContext *context;
 	GHashTable *args;
-#if WITH_GNOME
-	GnomeProgram *program;
-#else
-	char *accel_filename;
+	gboolean enable_metadata = FALSE;
 	GError *error = NULL;
-#endif
+
+	/* Init glib threads asap */
+	if (!g_thread_supported ())
+		g_thread_init (NULL);
 
 	context = g_option_context_new (_("GNOME Document Viewer"));
 
 #ifdef ENABLE_NLS
 	/* Initialize the i18n stuff */
-	bindtextdomain(GETTEXT_PACKAGE, GNOMELOCALEDIR);
-	bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
-	textdomain(GETTEXT_PACKAGE);
+	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
+	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+	textdomain (GETTEXT_PACKAGE);
 	g_option_context_add_main_entries (context, goption_options, GETTEXT_PACKAGE);
 	g_option_context_set_translation_domain(context, GETTEXT_PACKAGE);
 #else
 	g_option_context_add_main_entries (context, goption_options, NULL);
-#endif
-
-#if WITH_GNOME
-	program = gnome_program_init (PACKAGE, VERSION,
-				      LIBGNOMEUI_MODULE, argc, argv,
-				      GNOME_PARAM_GOPTION_CONTEXT, context,
-				      GNOME_PARAM_HUMAN_READABLE_NAME, _("Evince"),
-				      GNOME_PARAM_APP_DATADIR, GNOMEDATADIR,
-				      NULL);
-#else
+#endif /* ENABLE_NLS */
+	
+	g_option_context_add_group (context, egg_sm_client_get_option_group ());
 	g_option_context_add_group (context, gtk_get_option_group (TRUE));
+
+	gtk_init (&argc, &argv);
+
 	if (!g_option_context_parse (context, &argc, &argv, &error)) {
 		g_warning ("Cannot parse arguments: %s", error->message);
 		g_error_free (error);
+		g_option_context_free (context);
+		
 		return 1;
 	}
 	g_option_context_free (context);
 
-	accel_filename = g_build_filename (ev_dot_dir (), "accels", NULL);
-	gtk_accel_map_load (accel_filename);
-#endif
-
 	args = arguments_parse ();
 
 #ifdef ENABLE_DBUS
 	if (!ev_application_register_service (EV_APP)) {
 		if (load_files_remote (file_arguments, args)) {
 			g_hash_table_destroy (args);
-#if WITH_GNOME
-			g_object_unref (program);
-#endif
+
 			return 0;
 		}
 	} else {
 		enable_metadata = TRUE;
 	}
-#endif
-
-#if WITH_GNOME	
-	gnome_authentication_manager_init ();
-#endif
+#endif /* ENABLE_DBUS */
 
+	ev_debug_init ();
 	ev_backends_manager_init ();
 	
 	if (enable_metadata) {
 		ev_metadata_manager_init ();
 	}
 
-	ev_job_queue_init ();
-	g_set_application_name (_("Evince Document Viewer"));
-
 	ev_file_helpers_init ();
 	ev_stock_icons_init ();
-	gtk_window_set_default_icon_name ("evince");
+	
+	egg_set_desktop_file (GNOMEDATADIR "/applications/evince.desktop");
 
-	load_files (file_arguments, args);
+	if (!ev_application_load_session (EV_APP))
+		load_files (file_arguments, args);
 	g_hash_table_destroy (args);
 
 	gtk_main ();
 
-#if WITH_GNOME
-	gnome_accelerators_sync ();
-#else
-	gtk_accel_map_save (accel_filename);
-	g_free (accel_filename);
-#endif
-
 	ev_file_helpers_shutdown ();
 
 	if (enable_metadata) {
@@ -415,9 +390,7 @@ main (int argc, char *argv[])
 
 	ev_backends_manager_shutdown ();
 
-#if WITH_GNOME
- 	g_object_unref (program);
-#endif
-	
+	ev_debug_shutdown ();
+
 	return 0;
 }
